@@ -23,9 +23,7 @@
 #include "build-ident.h"
 #include "config.h"
 
-#ifdef HAVE_ENCHANT
-# include <enchant.h>
-#endif
+#include <libspelling.h>
 
 #include <glib/gi18n.h>
 
@@ -70,6 +68,7 @@ get_system_information (void)
   g_autoptr(GSettings) settings = g_settings_new ("org.gnome.TextEditor");
   g_autoptr(GSettingsSchema) schema = NULL;
   g_autofree char *theme_name = NULL;
+  const char *desktop_session;
   g_auto(GStrv) keys = NULL;
 
   g_object_get (gtk_settings, "gtk-theme-name", &theme_name, NULL);
@@ -117,18 +116,18 @@ get_system_information (void)
                           ADW_MINOR_VERSION,
                           ADW_MICRO_VERSION);
   g_string_append_printf (str,
-                          "Enchant2: %s\n",
-#ifdef HAVE_ENCHANT
-                          enchant_get_version ());
-#else
-                          "Unavailable");
-#endif
+                          "Libspelling: (%s)\n",
+                          SPELLING_VERSION_S);
 
   g_string_append (str, "\n");
   g_string_append_printf (str, "gtk-theme-name: %s\n", theme_name);
   g_string_append_printf (str, "GTK_THEME: %s\n", g_getenv ("GTK_THEME") ?: "unset");
   g_string_append_printf (str, "GdkDisplay: %s\n", G_OBJECT_TYPE_NAME (gdk_display_get_default ()));
 
+  if (!(desktop_session = g_getenv ("DESKTOP_SESSION")))
+    desktop_session = "missing";
+  g_string_append_printf (str, "DESKTOP_SESSION: %s\n", desktop_session);
+
   g_string_append (str, "\n");
   g_string_append_printf (str, "Documents Directory: %s\n", g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS));
 
@@ -208,7 +207,7 @@ editor_application_actions_about_cb (GSimpleAction *action,
                          "issue-url", PACKAGE_BUGREPORTS,
                          "developers", developers,
                          "designers", designers,
-                         "copyright", "© 2020-2023 Christian Hergert, et al.",
+                         "copyright", "© 2020-2025 Christian Hergert, et al.",
                          "documenters", documenters,
                          "license-type", GTK_LICENSE_GPL_3_0,
                          "translator-credits", _("translator-credits"),
@@ -222,14 +221,16 @@ editor_application_actions_help_cb (GSimpleAction *action,
                                     gpointer       user_data)
 {
   EditorApplication *self = user_data;
-  g_autoptr(GtkUriLauncher) launcher = NULL;
-  EditorWindow *window;
+  GdkAppLaunchContext *context = NULL;
+  g_autoptr(GError) error = NULL;
 
   g_assert (EDITOR_IS_APPLICATION (self));
 
-  window = editor_application_get_current_window (self);
-  launcher = gtk_uri_launcher_new ("help:gnome-text-editor");
-  gtk_uri_launcher_launch (launcher, GTK_WINDOW (window), NULL, NULL, NULL);
+  context = gdk_display_get_app_launch_context (gdk_display_get_default ());
+  if (!g_app_info_launch_default_for_uri ("help:gnome-text-editor",
+                                          G_APP_LAUNCH_CONTEXT (context),
+                                          &error))
+    g_warning ("Failed to launch help: %s", error->message);
 }
 
 static void
@@ -338,10 +339,28 @@ editor_application_actions_clear_history (GSimpleAction *action,
                                           gpointer       user_data)
 {
   EditorApplication *self = user_data;
+  EditorWindow *window;
+  AdwDialog *dialog;
 
   g_assert (EDITOR_IS_APPLICATION (self));
 
-  _editor_session_clear_history (self->session);
+  dialog = adw_alert_dialog_new (_("Clear File History?"),
+                                 _("This action cannot be undone"));
+  adw_alert_dialog_add_responses (ADW_ALERT_DIALOG (dialog),
+                                  "cancel", _("_Cancel"),
+                                  "clear-all", _("_Clear All"),
+                                  NULL);
+  adw_alert_dialog_set_response_appearance (ADW_ALERT_DIALOG (dialog),
+                                            "clear-all", ADW_RESPONSE_DESTRUCTIVE);
+
+  g_signal_connect_object (dialog,
+                           "response::clear-all",
+                           G_CALLBACK (_editor_session_clear_history),
+                           self->session,
+                           G_CONNECT_SWAPPED);
+
+  window = editor_application_get_current_window (self);
+  adw_dialog_present (dialog, GTK_WIDGET (window));
 }
 
 void
