@@ -75,6 +75,7 @@ struct _EditorDocument
   guint                         did_shutdown : 1;
   guint                         inserting : 1;
   guint                         deleting : 1;
+  guint                         supress_jump : 1;
 };
 
 typedef struct
@@ -585,7 +586,7 @@ editor_document_cursor_moved (EditorDocument *self)
 {
   g_assert (EDITOR_IS_DOCUMENT (self));
 
-  if (self->inserting || self->deleting)
+  if (self->inserting || self->deleting || self->supress_jump)
     return;
 
   g_signal_emit (self, signals[CURSOR_JUMPED], 0);
@@ -1257,7 +1258,8 @@ editor_document_save_cb (GObject      *object,
 
       /* Try again as admin:// */
       if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED) &&
-          g_str_has_prefix (uri, "file://"))
+          g_str_has_prefix (uri, "file://") &&
+          !g_object_get_data (G_OBJECT (saver), "IS_ADMIN"))
         {
           g_autofree char *admin_uri = g_strdup_printf ("admin://%s", uri + strlen ("file://"));
           g_autoptr(GFile) admin_file = g_file_new_for_uri (admin_uri);
@@ -1271,6 +1273,8 @@ editor_document_save_cb (GObject      *object,
           gtk_source_file_saver_set_flags (admin_saver, gtk_source_file_saver_get_flags (saver));
           gtk_source_file_saver_set_newline_type (admin_saver, gtk_source_file_saver_get_newline_type (saver));
 
+          g_object_set_data (G_OBJECT (admin_saver), "IS_ADMIN", GINT_TO_POINTER (TRUE));
+
           gtk_source_file_saver_save_async (admin_saver,
                                             G_PRIORITY_DEFAULT,
                                             NULL,
@@ -1451,7 +1455,9 @@ _editor_document_unmark_busy (EditorDocument *self)
        * wants to watch the cursor position can safely ignore changes while
        * the document is busy.
        */
+      self->supress_jump = TRUE;
       g_signal_emit_by_name (self, "cursor-moved");
+      self->supress_jump = FALSE;
     }
 }
 
