@@ -27,6 +27,7 @@
 #include "editor-page.h"
 #include "editor-page-settings.h"
 #include "editor-path-private.h"
+#include "editor-position-label-private.h"
 #include "editor-print-operation.h"
 #include "editor-search-bar-private.h"
 #include "editor-utils-private.h"
@@ -52,14 +53,17 @@ struct _EditorPage
   GtkSourceView           *view;
   GtkSourceMap            *map;
   GtkProgressBar          *progress_bar;
-  GtkRevealer             *goto_line_revealer;
+  GtkWidget               *goto_line_revealer;
   GtkEntry                *goto_line_entry;
-  GtkRevealer             *search_revealer;
+  AdwToolbarView          *toolbar_view;
+  GtkWidget               *search_revealer;
   EditorSearchBar         *search_bar;
   GtkInfoBar              *changed_infobar;
   GtkInfoBar              *infobar;
   GtkEventController      *vim;
-  GMenu                   *indent_menu;
+  EditorPositionLabel     *position_label;
+
+  guint                    queued_hide_position;
 
   guint                    close_requested : 1;
   guint                    moving : 1;
