@@ -22,8 +22,6 @@
 
 #include "config.h"
 
-#include <unistd.h>
-
 #include <glib/gi18n.h>
 
 #include <adwaita.h>
@@ -38,6 +36,8 @@
 #include "editor-source-view.h"
 #include "editor-utils-private.h"
 
+#define HIDE_POSITION_DELAY_SECONDS 1
+
 enum {
   PROP_0,
   PROP_BUSY,
@@ -57,6 +57,36 @@ enum {
 G_DEFINE_TYPE (EditorPage, editor_page, GTK_TYPE_WIDGET)
 
 static GParamSpec *properties [N_PROPS];
+static GSettings *app_settings;
+
+static gboolean
+editor_page_hide_position_cb (gpointer data)
+{
+  EditorPage *self = data;
+
+  self->queued_hide_position = 0;
+
+  _editor_widget_hide_with_fade_delay (GTK_WIDGET (self->position_label), 500);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+editor_page_queue_hide_position (EditorPage *self)
+{
+  g_assert (EDITOR_IS_PAGE (self));
+
+  if (self->queued_hide_position)
+    g_source_remove (self->queued_hide_position);
+
+  if (!g_settings_get_boolean (app_settings, "show-line-numbers"))
+    return;
+
+  self->queued_hide_position =
+    g_timeout_add_seconds (HIDE_POSITION_DELAY_SECONDS,
+                           editor_page_hide_position_cb,
+                           self);
+}
 
 static gboolean
 get_tab_info (EditorPage  *self,
@@ -253,7 +283,35 @@ editor_page_document_cursor_moved_cb (EditorPage     *self,
   g_assert (EDITOR_IS_DOCUMENT (document));
 
   if (!_editor_document_get_loading (document))
-    g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_POSITION_LABEL]);
+    {
+      GtkTextMark *insert = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (document));
+      GtkTextIter iter;
+
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_POSITION_LABEL]);
+
+      gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (document), &iter, insert);
+
+      _editor_position_label_set_position (self->position_label,
+                                           gtk_text_iter_get_line (&iter) + 1,
+                                           gtk_text_iter_get_line_offset (&iter) + 1);
+    }
+}
+
+static void
+editor_page_document_cursor_jumped_cb (EditorPage     *self,
+                                       EditorDocument *document)
+{
+  g_assert (EDITOR_IS_PAGE (self));
+  g_assert (EDITOR_IS_DOCUMENT (document));
+
+  if (g_settings_get_boolean (app_settings, "show-line-numbers"))
+    {
+      g_object_set_data (G_OBJECT (self->position_label),
+                         "EDITOR_FADE_ANIMATION",
+                         NULL);
+      gtk_widget_show (GTK_WIDGET (self->position_label));
+      editor_page_queue_hide_position (self);
+    }
 }
 
 static void
@@ -358,6 +416,11 @@ editor_page_set_document (EditorPage     *self,
                                G_CALLBACK (editor_page_document_cursor_moved_cb),
                                self,
                                G_CONNECT_SWAPPED);
+      g_signal_connect_object (document,
+                               "cursor-jumped",
+                               G_CALLBACK (editor_page_document_cursor_jumped_cb),
+                               self,
+                               G_CONNECT_SWAPPED);
       g_signal_connect_object (document,
                                "notify::file",
                                G_CALLBACK (editor_page_document_notify_file_cb),
@@ -606,30 +669,20 @@ get_child_position_cb (GtkOverlay    *overlay,
   g_assert (alloc != NULL);
   g_assert (EDITOR_IS_PAGE (self));
 
-  if (child != GTK_WIDGET (self->search_revealer) ||
-      !gtk_widget_get_visible (GTK_WIDGET (self->map)))
+  if (child != GTK_WIDGET (self->position_label))
+    return FALSE;
+
+  if (!gtk_widget_get_visible (GTK_WIDGET (self->map)))
     return FALSE;
 
   gtk_widget_get_allocation (GTK_WIDGET (overlay), &ov_alloc);
   gtk_widget_get_preferred_size (child, &min, &nat);
   gtk_widget_get_preferred_size (GTK_WIDGET (self->map), &map_min, NULL);
 
-  if (nat.width + map_min.width <= ov_alloc.width)
-    {
-      alloc->x = ov_alloc.width;
-      alloc->x -= map_min.width;
-      alloc->x -= nat.width;
-      alloc->width = nat.width;
-      alloc->y = 0;
-      alloc->height = min.height;
-    }
-  else
-    {
-      alloc->x = 0;
-      alloc->width = min.width;
-      alloc->y = 0;
-      alloc->height = min.height;
-    }
+  alloc->x = ov_alloc.width - map_min.width - nat.width;
+  alloc->y = ov_alloc.height - nat.height;
+  alloc->width = nat.width;
+  alloc->height = nat.height;
 
   return TRUE;
 }
@@ -691,6 +744,8 @@ editor_page_dispose (GObject *object)
 {
   EditorPage *self = (EditorPage *)object;
 
+  g_clear_handle_id (&self->queued_hide_position, g_source_remove);
+
   if (self->document != NULL)
     _editor_document_shutdown (self->document);
 
@@ -895,16 +950,17 @@ editor_page_class_init (EditorPageClass *klass)
   gtk_widget_class_set_css_name (widget_class, "page");
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/TextEditor/ui/editor-page.ui");
   gtk_widget_class_bind_template_child (widget_class, EditorPage, box);
-  gtk_widget_class_bind_template_child (widget_class, EditorPage, goto_line_entry);
   gtk_widget_class_bind_template_child (widget_class, EditorPage, goto_line_revealer);
-  gtk_widget_class_bind_template_child (widget_class, EditorPage, indent_menu);
+  gtk_widget_class_bind_template_child (widget_class, EditorPage, goto_line_entry);
   gtk_widget_class_bind_template_child (widget_class, EditorPage, infobar);
   gtk_widget_class_bind_template_child (widget_class, EditorPage, map);
   gtk_widget_class_bind_template_child (widget_class, EditorPage, overlay);
+  gtk_widget_class_bind_template_child (widget_class, EditorPage, position_label);
   gtk_widget_class_bind_template_child (widget_class, EditorPage, progress_bar);
   gtk_widget_class_bind_template_child (widget_class, EditorPage, scroller);
   gtk_widget_class_bind_template_child (widget_class, EditorPage, search_bar);
   gtk_widget_class_bind_template_child (widget_class, EditorPage, search_revealer);
+  gtk_widget_class_bind_template_child (widget_class, EditorPage, toolbar_view);
   gtk_widget_class_bind_template_child (widget_class, EditorPage, view);
   gtk_widget_class_bind_template_callback (widget_class, get_child_position_cb);
   gtk_widget_class_bind_template_callback (widget_class, goto_line_entry_activate_cb);
@@ -914,6 +970,7 @@ editor_page_class_init (EditorPageClass *klass)
   gtk_widget_class_add_binding_action (widget_class, GDK_KEY_i, GDK_CONTROL_MASK, "page.show-goto-line", NULL);
 
   g_type_ensure (EDITOR_TYPE_INFO_BAR);
+  g_type_ensure (EDITOR_TYPE_POSITION_LABEL);
   g_type_ensure (EDITOR_TYPE_SEARCH_BAR);
   g_type_ensure (EDITOR_TYPE_SOURCE_MAP);
   g_type_ensure (EDITOR_TYPE_SOURCE_VIEW);
@@ -924,15 +981,13 @@ editor_page_init (EditorPage *self)
 {
   GtkDropTarget *dest;
 
+  if (app_settings == NULL)
+    app_settings = g_settings_new ("org.gnome.TextEditor");
+
   self->cancellable = g_cancellable_new ();
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  editor_source_view_prepend_extra_menu (EDITOR_SOURCE_VIEW (self->view),
-                                         G_MENU_MODEL (self->indent_menu));
-  _editor_revealer_auto_hide (self->search_revealer);
-  _editor_revealer_auto_hide (self->goto_line_revealer);
-
   /* Work around https://gitlab.gnome.org/GNOME/gtk/-/issues/4315
    * by connecting to the GtkText to intercept insert-text() emission.
    */
@@ -1091,7 +1146,7 @@ editor_page_dup_subtitle (EditorPage *self)
 
 #ifdef G_OS_UNIX
   if (docportal == NULL)
-    docportal = g_strdup_printf ("/run/user/%u/doc/", getuid ());
+    docportal = g_strdup_printf ("%s/doc/", g_get_user_runtime_dir ());
 #endif
 
   file = editor_document_get_file (self->document);
@@ -1119,7 +1174,7 @@ editor_page_dup_subtitle (EditorPage *self)
 
 #ifdef G_OS_UNIX
   if (g_str_has_prefix (peek, docportal))
-    return g_strdup (_("Document Portal"));
+    return _editor_get_portal_host_path (dir);
 #endif
 
   return _editor_path_collapse (peek);
@@ -1191,55 +1246,80 @@ _editor_page_save (EditorPage *self)
                                g_object_ref (self));
 }
 
-static void
-editor_page_save_as_cb (EditorPage           *self,
-                        gint                  response_id,
-                        GtkFileChooserNative *native)
+static GtkSourceNewlineType
+parse_line_ending (const char *str)
 {
+  if (*str == '\n')
+    return GTK_SOURCE_NEWLINE_TYPE_LF;
+
+  if (str[0] == '\r' && str[1] == '\n')
+    return GTK_SOURCE_NEWLINE_TYPE_CR_LF;
+
+  if (*str == '\r')
+    return GTK_SOURCE_NEWLINE_TYPE_CR;
+
+  return 0;
+}
+
+static const GtkSourceEncoding *
+parse_encoding (const char *str)
+{
+  if (g_strcmp0 (str, "") == 0 || g_strcmp0 (str, "automatic") == 0)
+    return NULL;
+
+  return gtk_source_encoding_get_from_charset (str);
+}
+
+static void
+editor_page_save_as_cb (GObject      *object,
+                        GAsyncResult *result,
+                        gpointer      user_data)
+{
+  GtkFileDialog *dialog = (GtkFileDialog *)object;
+  g_autoptr(EditorPage) self = user_data;
+  g_autoptr(GError) error = NULL;
+  g_autoptr(GFile) dest = NULL;
+  const char *encoding = NULL;
+  const char *line_endings = NULL;
+
+  g_assert (GTK_IS_FILE_DIALOG (dialog));
+  g_assert (G_IS_ASYNC_RESULT (result));
   g_assert (EDITOR_IS_PAGE (self));
-  g_assert (GTK_IS_FILE_CHOOSER_NATIVE (native));
 
-  if (response_id == GTK_RESPONSE_ACCEPT)
+  if (!(dest = gtk_file_dialog_save_text_file_finish (dialog, result, &encoding, &line_endings, &error)))
     {
-      g_autoptr(GFile) dest = NULL;
-
-      if ((dest = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (native))))
-        {
-          const GtkSourceEncoding *encoding = _editor_file_chooser_get_encoding (GTK_FILE_CHOOSER (native));
-          GtkSourceNewlineType crlf = _editor_file_chooser_get_line_ending (GTK_FILE_CHOOSER (native));
-          g_autoptr(GFile) directory = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (native));
-
-          if (directory == NULL)
-            directory = g_file_get_parent (dest);
-
-          if (directory != NULL)
-            {
-              g_autoptr(GSettings) settings = g_settings_new ("org.gnome.TextEditor");
-              g_autofree char *uri = g_file_get_uri (directory);
-              g_settings_set_string (settings, "last-save-directory", uri);
-            }
-
-          _editor_document_set_encoding (self->document, encoding);
-          _editor_document_set_newline_type (self->document, crlf);
-          _editor_document_save_async (self->document,
-                                       dest,
-                                       NULL,
-                                       editor_page_save_cb,
-                                       g_object_ref (self));
-        }
+      g_warning ("%s", error->message);
+      return;
     }
 
-  gtk_native_dialog_destroy (GTK_NATIVE_DIALOG (native));
-  g_object_unref (native);
+  {
+    g_autoptr(GFile) directory = g_file_get_parent (dest);
+    g_autoptr(GSettings) settings = g_settings_new ("org.gnome.TextEditor");
+    g_autofree char *uri = g_file_get_uri (directory);
+
+    g_settings_set_string (settings, "last-save-directory", uri);
+  }
+
+  if (encoding != NULL)
+    _editor_document_set_encoding (self->document, parse_encoding (encoding));
+
+  if (line_endings != NULL && line_endings[0])
+    _editor_document_set_newline_type (self->document, parse_line_ending (line_endings));
+
+  _editor_document_save_async (self->document,
+                               dest,
+                               NULL,
+                               editor_page_save_cb,
+                               g_object_ref (self));
 }
 
 void
 _editor_page_save_as (EditorPage *self,
                       const char *filename)
 {
+  g_autoptr(GtkFileDialog) dialog = NULL;
   g_autoptr(GSettings) settings = NULL;
   g_autoptr(GFile) directory = NULL;
-  GtkFileChooserNative *native;
   EditorWindow *window;
   g_autofree char *uri = NULL;
   GFile *file;
@@ -1254,16 +1334,7 @@ _editor_page_save_as (EditorPage *self,
   _editor_page_raise (self);
 
   window = _editor_page_get_window (self);
-  native = gtk_file_chooser_native_new (_("Save As"),
-                                        GTK_WINDOW (window),
-                                        GTK_FILE_CHOOSER_ACTION_SAVE,
-                                        _("Save"),
-                                        _("Cancel"));
-
-  _editor_file_chooser_add_encodings (GTK_FILE_CHOOSER (native),
-                                      _editor_document_get_encoding (self->document));
-  _editor_file_chooser_add_line_endings (GTK_FILE_CHOOSER (native),
-                                         _editor_document_get_newline_type (self->document));
+  dialog = gtk_file_dialog_new ();
 
   if (filename != NULL)
     {
@@ -1281,32 +1352,31 @@ _editor_page_save_as (EditorPage *self,
           selected = g_file_new_for_path (filename);
         }
 
-      gtk_file_chooser_set_file (GTK_FILE_CHOOSER (native), selected, NULL);
+      gtk_file_dialog_set_initial_file (dialog, selected);
     }
   else if ((file = editor_document_get_file (self->document)))
     {
       g_autoptr(GFile) parent = g_file_get_parent (file);
 
-      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (native), parent, NULL);
+      gtk_file_dialog_set_initial_folder (dialog, parent);
     }
   else if (directory != NULL)
     {
-      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (native), directory, NULL);
+      gtk_file_dialog_set_initial_folder (dialog, directory);
     }
 
   if (filename == NULL)
     {
       g_autofree char *suggestion = _editor_document_suggest_filename (self->document);
-      gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (native), suggestion);
+
+      gtk_file_dialog_set_initial_name (dialog, suggestion);
     }
 
-  g_signal_connect_object (native,
-                           "response",
-                           G_CALLBACK (editor_page_save_as_cb),
-                           self,
-                           G_CONNECT_SWAPPED);
-
-  gtk_native_dialog_show (GTK_NATIVE_DIALOG (native));
+  gtk_file_dialog_save_text_file (dialog,
+                                  GTK_WINDOW (window),
+                                  NULL,
+                                  editor_page_save_as_cb,
+                                  g_object_ref (self));
 }
 
 gboolean
@@ -1499,8 +1569,8 @@ _editor_page_set_search_visible (EditorPage          *self,
       _editor_search_bar_detach (self->search_bar);
     }
 
-  gtk_revealer_set_reveal_child (self->goto_line_revealer, FALSE);
-  gtk_revealer_set_reveal_child (self->search_revealer, search_visible);
+  gtk_revealer_set_reveal_child (GTK_REVEALER (self->goto_line_revealer), FALSE);
+  gtk_revealer_set_reveal_child (GTK_REVEALER (self->search_revealer), search_visible);
 
   if (search_visible)
     _editor_search_bar_grab_focus (self->search_bar);
@@ -1533,12 +1603,12 @@ _editor_page_hide_search (EditorPage *self)
 void
 _editor_page_scroll_to_insert (EditorPage *self)
 {
-  GtkTextMark *mark;
-
   g_return_if_fail (EDITOR_IS_PAGE (self));
 
-  mark = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (self->document));
-  gtk_text_view_scroll_mark_onscreen (GTK_TEXT_VIEW (self->view), mark);
+  gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (self->view),
+                                gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (self->document)),
+                                0.1,
+                                FALSE, 0, 0);
 }
 
 void
@@ -1769,3 +1839,4 @@ editor_page_destroy (EditorPage *self)
 
   g_object_run_dispose (G_OBJECT (self));
 }
+
