@@ -28,6 +28,7 @@
 #include "editor-document.h"
 #include "editor-open-popover-private.h"
 #include "editor-page-private.h"
+#include "editor-properties-panel.h"
 #include "editor-save-changes-dialog-private.h"
 #include "editor-session-private.h"
 #include "editor-theme-selector-private.h"
@@ -180,58 +181,6 @@ update_subtitle_visibility_cb (EditorWindow *self)
     }
 
   gtk_widget_set_visible (GTK_WIDGET (self->subtitle), visible);
-  gtk_widget_set_visible (GTK_WIDGET (self->position_box),
-                          visible && g_settings_get_boolean (self->settings, "show-line-numbers"));
-}
-
-static void
-update_language_name_cb (EditorWindow *self)
-{
-  const char *language_name = NULL;
-  const char *label = _("Document Type");
-  g_autofree char *freeme = NULL;
-  EditorPage *page;
-
-  g_assert (EDITOR_IS_WINDOW (self));
-
-  if ((page = editor_window_get_visible_page (self)))
-    language_name = editor_page_get_language_name (page);
-
-  g_menu_remove (G_MENU (self->doc_type_menu), self->doc_type_index);
-
-   if (language_name)
-     label = freeme = g_strdup_printf (_("Document Type: %s"), language_name);
-
-   g_menu_insert (G_MENU (self->doc_type_menu),
-                  self->doc_type_index,
-                  label,
-                  "page.change-language");
-}
-
-static void
-editor_window_cursor_moved_cb (EditorWindow   *self,
-                               EditorDocument *document)
-{
-  EditorPage *page;
-  guint line;
-  guint column;
-
-  g_assert (EDITOR_IS_WINDOW (self));
-  g_assert (self->position_label != NULL);
-  g_assert (EDITOR_IS_POSITION_LABEL (self->position_label));
-  g_assert (EDITOR_IS_DOCUMENT (document));
-
-  if (!(page = editor_window_get_visible_page (self)))
-    return;
-
-  if (editor_document_get_busy (document))
-    return;
-
-  if (editor_page_get_document (page) != document)
-    return;
-
-  editor_page_get_visual_position (page, &line, &column);
-  _editor_position_label_set_position (self->position_label, line + 1, column + 1);
 }
 
 static void
@@ -239,21 +188,13 @@ editor_window_page_bind_cb (EditorWindow *self,
                             EditorPage   *page,
                             GSignalGroup *group)
 {
-  EditorDocument *document;
-
   g_assert (EDITOR_IS_WINDOW (self));
   g_assert (EDITOR_IS_PAGE (page));
   g_assert (G_IS_SIGNAL_GROUP (group));
 
-  document = editor_page_get_document (page);
-
-  g_signal_group_set_target (self->document_signals, document);
-  editor_window_cursor_moved_cb (self, document);
-
   _editor_window_actions_update (self, page);
 
   update_subtitle_visibility_cb (self);
-  update_language_name_cb (self);
 }
 
 static void
@@ -295,12 +236,9 @@ editor_window_notify_selected_page_cb (EditorWindow *self,
 
   gtk_label_set_label (self->title, _(PACKAGE_NAME));
   gtk_label_set_label (self->subtitle, NULL);
-  gtk_label_set_label (GTK_LABEL (self->zoom_label), "100%");
   gtk_widget_set_sensitive (self->zoom_label, page != NULL);
   gtk_widget_set_visible (GTK_WIDGET (self->is_modified), FALSE);
   gtk_widget_set_visible (GTK_WIDGET (self->subtitle), FALSE);
-  gtk_widget_set_visible (GTK_WIDGET (self->position_box),
-                          page && g_settings_get_boolean (self->settings, "show-line-numbers"));
   gtk_widget_set_visible (GTK_WIDGET (self->indicator), FALSE);
 
   self->visible_page = page;
@@ -321,6 +259,8 @@ editor_window_notify_selected_page_cb (EditorWindow *self,
    *        go away.
    */
   gtk_widget_queue_resize (GTK_WIDGET (self->tab_view));
+
+  editor_fullscreen_box_reveal (self->fullscreen_box);
 }
 
 static void
@@ -334,9 +274,9 @@ editor_window_items_changed_cb (EditorWindow      *self,
   g_assert (GTK_IS_SELECTION_MODEL (model));
 
   if (editor_window_get_n_pages (self) > 0)
-    gtk_stack_set_visible_child (self->stack, GTK_WIDGET (self->tab_view));
+    gtk_stack_set_visible_child_name (self->stack, "tabs");
   else
-    gtk_stack_set_visible_child (self->stack, GTK_WIDGET (self->empty));
+    gtk_stack_set_visible_child_name (self->stack, "empty");
 }
 
 static void
@@ -413,48 +353,6 @@ editor_window_close_request (GtkWindow *window)
   return GTK_WINDOW_CLASS (editor_window_parent_class)->close_request (window);
 }
 
-static gboolean
-find_document_type (GMenuModel  *menu,
-                    GMenuModel **container,
-                    guint       *index)
-{
-  guint n_items;
-
-  g_assert (G_IS_MENU_MODEL (menu));
-  g_assert (container != NULL && *container == NULL);
-  g_assert (index != NULL && *index == 0);
-
-  n_items = g_menu_model_get_n_items (menu);
-
-  for (guint i = 0; i < n_items; i++)
-    {
-      g_autoptr(GMenuLinkIter) iter = NULL;
-      g_autofree char *action = NULL;
-
-      if (g_menu_model_get_item_attribute (menu, i, "action", "s", &action))
-        {
-          if (g_strcmp0 (action, "page.change-language") == 0)
-            {
-              *container = g_object_ref (menu);
-              *index = i;
-              return TRUE;
-            }
-        }
-
-      iter = g_menu_model_iterate_item_links (menu, i);
-
-      while (g_menu_link_iter_next (iter))
-        {
-          g_autoptr(GMenuModel) link = g_menu_link_iter_get_value (iter);
-
-          if (find_document_type (link, container, index))
-            return TRUE;
-        }
-    }
-
-  return FALSE;
-}
-
 static void
 editor_window_constructed (GObject *object)
 {
@@ -471,11 +369,6 @@ editor_window_constructed (GObject *object)
 
   session = editor_application_get_session (EDITOR_APPLICATION_DEFAULT);
 
-  find_document_type (G_MENU_MODEL (self->options_menu_model),
-                      &self->doc_type_menu,
-                      &self->doc_type_index);
-  g_assert (G_IS_MENU_MODEL (self->doc_type_menu));
-
   /* Set the recents list for the open popover */
   g_object_bind_property (session, "recents",
                           self->open_menu_popover, "model",
@@ -508,10 +401,10 @@ editor_window_constructed (GObject *object)
                                   _("Zoom in"), -1);
   zoom_out = g_object_new (GTK_TYPE_BUTTON,
                            "action-name", "page.zoom-out",
-                          "child", g_object_new (GTK_TYPE_IMAGE,
-                                                 "icon-name", "zoom-out-symbolic",
-                                                 "pixel-size", 16,
-                                                 NULL),
+                           "child", g_object_new (GTK_TYPE_IMAGE,
+                                                  "icon-name", "zoom-out-symbolic",
+                                                  "pixel-size", 16,
+                                                  NULL),
                            NULL);
   gtk_widget_add_css_class (zoom_out, "circular");
   gtk_widget_add_css_class (zoom_out, "flat");
@@ -519,10 +412,14 @@ editor_window_constructed (GObject *object)
   gtk_accessible_update_property (GTK_ACCESSIBLE (zoom_out),
                                   GTK_ACCESSIBLE_PROPERTY_LABEL,
                                   _("Zoom out"), -1);
-  self->zoom_label = g_object_new (GTK_TYPE_LABEL,
+  self->zoom_label = g_object_new (GTK_TYPE_BUTTON,
+                                   "css-classes", (const char * const[]) {"flat", "pill", NULL},
+                                   "action-name", "page.zoom-one",
                                    "hexpand", TRUE,
+                                   "tooltip-text", _("Reset Zoom"),
                                    "label", "100%",
                                    NULL);
+
   g_binding_group_bind (self->page_bindings, "zoom-label", self->zoom_label, "label", 0);
   gtk_box_append (GTK_BOX (zoom_box), zoom_out);
   gtk_box_append (GTK_BOX (zoom_box), self->zoom_label);
@@ -827,6 +724,70 @@ indicator_to_boolean (GBinding     *binding,
   return TRUE;
 }
 
+static void
+editor_window_fullscreen_action (GtkWidget  *widget,
+                                 const char *action_name,
+                                 GVariant   *param)
+{
+  gtk_window_fullscreen (GTK_WINDOW (widget));
+}
+
+static void
+editor_window_unfullscreen_action (GtkWidget  *widget,
+                                   const char *action_name,
+                                   GVariant   *param)
+{
+  gtk_window_unfullscreen (GTK_WINDOW (widget));
+}
+
+static void
+editor_window_toggle_fullscreen (GtkWidget  *widget,
+                                 const char *action_name,
+                                 GVariant   *param)
+{
+  if (gtk_window_is_fullscreen (GTK_WINDOW (widget)))
+    gtk_window_unfullscreen (GTK_WINDOW (widget));
+  else
+    gtk_window_fullscreen (GTK_WINDOW (widget));
+}
+
+static void
+editor_window_toplevel_state_changed_cb (EditorWindow *self,
+                                         GParamSpec   *pspec,
+                                         GdkToplevel  *toplevel)
+{
+  GdkToplevelState state;
+  gboolean is_fullscreen;
+
+  g_assert (EDITOR_IS_WINDOW (self));
+  g_assert (pspec != NULL);
+  g_assert (GDK_IS_TOPLEVEL (toplevel));
+
+  state = gdk_toplevel_get_state (toplevel);
+
+  is_fullscreen = !!(state & GDK_TOPLEVEL_STATE_FULLSCREEN);
+
+  gtk_widget_action_set_enabled (GTK_WIDGET (self), "win.fullscreen", !is_fullscreen);
+  gtk_widget_action_set_enabled (GTK_WIDGET (self), "win.unfullscreen", is_fullscreen);
+
+  editor_fullscreen_box_set_fullscreen (self->fullscreen_box, is_fullscreen);
+}
+
+static void
+editor_window_realize (GtkWidget *widget)
+{
+  GdkSurface *toplevel;
+
+  GTK_WIDGET_CLASS (editor_window_parent_class)->realize (widget);
+
+  if ((toplevel = gtk_native_get_surface (GTK_NATIVE (widget))))
+    g_signal_connect_object (toplevel,
+                             "notify::state",
+                             G_CALLBACK (editor_window_toplevel_state_changed_cb),
+                             widget,
+                             G_CONNECT_SWAPPED);
+}
+
 static void
 editor_window_dispose (GObject *object)
 {
@@ -840,7 +801,6 @@ editor_window_dispose (GObject *object)
 
   g_binding_group_set_source (self->page_bindings, NULL);
   g_signal_group_set_target (self->page_signals, NULL);
-  g_signal_group_set_target (self->document_signals, NULL);
 
   if (self->inhibit_cookie != 0)
     {
@@ -861,7 +821,6 @@ editor_window_finalize (GObject *object)
 
   g_clear_object (&self->page_bindings);
   g_clear_object (&self->page_signals);
-  g_clear_object (&self->document_signals);
   g_clear_object (&self->cancellable);
   g_clear_pointer (&self->closed_items, g_array_unref);
 
@@ -919,6 +878,8 @@ editor_window_class_init (EditorWindowClass *klass)
   object_class->get_property = editor_window_get_property;
   object_class->set_property = editor_window_set_property;
 
+  widget_class->realize = editor_window_realize;
+
   window_class->close_request = editor_window_close_request;
 
   properties [PROP_VISIBLE_PAGE] =
@@ -933,14 +894,11 @@ editor_window_class_init (EditorWindowClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/TextEditor/ui/editor-window.ui");
 
   gtk_widget_class_bind_template_child (widget_class, EditorWindow, empty);
+  gtk_widget_class_bind_template_child (widget_class, EditorWindow, fullscreen_box);
   gtk_widget_class_bind_template_child (widget_class, EditorWindow, indicator);
   gtk_widget_class_bind_template_child (widget_class, EditorWindow, is_modified);
   gtk_widget_class_bind_template_child (widget_class, EditorWindow, open_menu_button);
   gtk_widget_class_bind_template_child (widget_class, EditorWindow, open_menu_popover);
-  gtk_widget_class_bind_template_child (widget_class, EditorWindow, options_menu);
-  gtk_widget_class_bind_template_child (widget_class, EditorWindow, options_menu_model);
-  gtk_widget_class_bind_template_child (widget_class, EditorWindow, position_box);
-  gtk_widget_class_bind_template_child (widget_class, EditorWindow, position_label);
   gtk_widget_class_bind_template_child (widget_class, EditorWindow, primary_menu);
   gtk_widget_class_bind_template_child (widget_class, EditorWindow, stack);
   gtk_widget_class_bind_template_child (widget_class, EditorWindow, subtitle);
@@ -956,6 +914,9 @@ editor_window_class_init (EditorWindowClass *klass)
 
   gtk_widget_class_install_action (widget_class, "win.alternate-help-overlay", NULL, on_show_help_overlay_cb);
   gtk_widget_class_install_action (widget_class, "win.undo-close-page", NULL, on_undo_close_page_cb);
+  gtk_widget_class_install_action (widget_class, "win.fullscreen", NULL, editor_window_fullscreen_action);
+  gtk_widget_class_install_action (widget_class, "win.unfullscreen", NULL, editor_window_unfullscreen_action);
+  gtk_widget_class_install_action (widget_class, "win.toggle-fullscreen", NULL, editor_window_toggle_fullscreen);
 
   gtk_widget_class_add_binding_action (widget_class, GDK_KEY_w, GDK_CONTROL_MASK, "win.close-page-or-window", NULL);
   gtk_widget_class_add_binding_action (widget_class, GDK_KEY_o, GDK_CONTROL_MASK, "win.open", NULL);
@@ -992,8 +953,10 @@ editor_window_class_init (EditorWindowClass *klass)
 
   _editor_window_class_actions_init (klass);
 
+  g_type_ensure (EDITOR_TYPE_FULLSCREEN_BOX);
   g_type_ensure (EDITOR_TYPE_OPEN_POPOVER);
   g_type_ensure (EDITOR_TYPE_POSITION_LABEL);
+  g_type_ensure (EDITOR_TYPE_PROPERTIES_PANEL);
   g_type_ensure (EDITOR_TYPE_STATUSBAR);
 }
 
@@ -1070,6 +1033,11 @@ editor_window_init (EditorWindow *self)
                                  G_CALLBACK (editor_window_update_actions),
                                  self,
                                  G_CONNECT_SWAPPED);
+  g_signal_group_connect_object (self->page_signals,
+                                 "notify::zoom-label",
+                                 G_CALLBACK (editor_window_update_actions),
+                                 self,
+                                 G_CONNECT_SWAPPED);
   g_signal_group_connect_object (self->page_signals,
                                  "notify::can-discard",
                                  G_CALLBACK (update_subtitle_visibility_cb),
@@ -1080,19 +1048,6 @@ editor_window_init (EditorWindow *self)
                                  G_CALLBACK (update_subtitle_visibility_cb),
                                  self,
                                  G_CONNECT_SWAPPED);
-  g_signal_group_connect_object (self->page_signals,
-                                 "notify::language-name",
-                                 G_CALLBACK (update_language_name_cb),
-                                 self,
-                                 G_CONNECT_SWAPPED);
-
-  self->document_signals = g_signal_group_new (EDITOR_TYPE_DOCUMENT);
-
-  g_signal_group_connect_object (self->document_signals,
-                                 "cursor-moved",
-                                 G_CALLBACK (editor_window_cursor_moved_cb),
-                                 self,
-                                 G_CONNECT_SWAPPED);
 
   self->page_bindings = g_binding_group_new ();
 
